@@ -169,7 +169,7 @@ def update_available(noreport=False):
               "| egrep '^(Package|Version):' " +\
               "| tr '\n' ' '" +\
               "| perl -p -e 's|Package: |\n|g; s|Version: ||g'" +\
-              "| sort | tail +2 | sed 's| $||' > " + available_file
+              "| sort | tail -n +2 | sed 's| $||' > " + available_file
     # Use langC in the following since it uses a grep.
     perform.execute(command, noquiet=True, langC=True) # root is not required.
     os.rename(temporary_file, previous_file)
