@@ -35,6 +35,7 @@ signal.signal(signal.SIGPIPE,signal.SIG_DFL)
 #
 # APT module
 #
+# import apt
 import apt_pkg
 
 #
@@ -89,6 +90,67 @@ def ping_host(hostname):
              JIG will continue to try to get the information required.
 """
 
+def get_available(command="dumpavail"):
+    "Return an apt_pkg object that represents the parsed list of packages."
+    
+    # This originally ran the apt-cache command as a pipe to
+    # ParseTagFile as in:
+    #  command = "apt-cache dumpavail"
+    #  packages_pipe = perform.execute(command, noquiet=True, pipe=True)
+    #  avail = apt_pkg.ParseTagFile(packages_pipe)
+    #    
+    # But as in Bug#366678 ParseTagFile no longer works in a pipe
+    # because of changes to apt_pkg by Michael Vogt. He supplied
+    # suggested fixes, but they were not tested and I've not had
+    # a chance to understand the new APT package for Python. So put
+    # the apt-cache dump into a file and then use that file here.
+
+    tmpcache = tempfile.mktemp()
+    command = "apt-cache dumpavail > " + tmpcache
+    perform.execute(command)
+    avail = apt_pkg.ParseTagFile(file(tmpcache))
+    if os.path.exists(tmpcache): os.remove(tmpcache)
+    return(avail)
+
+def do_dependents(package):
+    command = "apt-cache showpkg " + package +\
+              " | awk \'BEGIN{found=0}/^Reverse Depends:/{found=1;next}/^[^ ]/{found=0}found==1{print}{next}\' " +\
+              " | sed \'s|Reverse Depends:||\' | tr \",\" \" \" " +\
+              " | awk \'{print $1}\' | sort -u | grep -v \'^$\'"
+    packages = perform.execute(command, pipe = True)
+    pkgs = []
+    for pkg in packages:
+        pkgs.append(pkg.strip())
+    avail = get_available()
+    #
+    # Check for information in the Available list
+    #
+    while avail.Step():
+        nam = avail.Section.get("Package")
+        if (nam in pkgs):
+            dep = avail.Section.get("Depends")
+            sug = avail.Section.get("Suggests")
+            rec = avail.Section.get("Recommends")
+            con = avail.Section.get("Conflicts")
+            rep = avail.Section.get("Replaces")
+            if not dep: dep = ""
+            if not sug: sug = ""
+            if not rec: rec = ""
+            if not con: con = ""
+            if not rep: rep = ""
+            if package in dep:
+                print "d", nam # depends
+            elif package in sug:
+                print "s", nam # suggests
+            elif package in rec:
+                print "r", nam # recommends
+            elif package in con:
+                print "c", nam # conflicts
+            elif package in rep:
+                print "p", nam # replaces
+            else:
+                print "o", nam # other
+
 #------------------------------------------------------------------------
 #
 # DESCRIBE
@@ -163,9 +225,8 @@ def do_describe(packages):
         packages = package_names
     else:
         return
-    command = "apt-cache dumpavail"
-    packages_pipe = perform.execute(command, noquiet=True, pipe=True)
-    avail = apt_pkg.ParseTagFile(packages_pipe)
+    avail = get_available()
+
     #
     # Record the descriptions
     #
@@ -179,6 +240,15 @@ def do_describe(packages):
             package_description = avail.Section.get("Description")
             if not describe_list.has_key(package_name):
                 describe_list[package_name] = package_description
+    # Bug fix for Bug#366678 Part 2 - not working yet???
+    # mvo: its probably a good idea to keep the cache around in all of wajig
+    #      because getting it may be expensive
+#     cache = apt.Cache()
+#     for pkgname in packages:
+#         if cache.has_key(pkgname):
+#             describe_list[pkgname] = cache[pkgname].description
+    # End Bug Fix for Bug#366678
+
     #
     # Print out the one line descriptions. Should it be sorted?
     # If not sorted it should be same order as on the command line.
@@ -211,6 +281,7 @@ def do_describe(packages):
 #        command += ") | sort"
 #        perform.execute(command)
         
+        # TODO mvo suggests apt.Package.summary may be helpful here
         for pkg in pkgs:
             # Only print that first line, but check that there
             # is a description available.
@@ -227,16 +298,27 @@ def do_describe(packages):
         # TODO is there a way of doing this using apt_pkg easily?
         # Otherwise "apt-cache show" seems okay if a little slower.
         #
+# Comment out for fix for Bug#366678 
         package_names = perform.concat(packages)
         command = "apt-cache show " + package_names
         perform.execute(command)
+# Bug#366678 fix from mvo - part 3 - not working yet???
+#         for pkgname in filter(lambda pkgname: cache.has_key(pkgname), packages):
+#             pkg = cache[pkgname]
+#             for ver in pkg._pkg.VersionList:
+#                 if ver == None or ver.FileList == None:
+#                     print "no ver or ver.FileList"
+#                     continue
+#                 f, index = ver.FileList.pop(0)
+#                 cache._records.Lookup((f,index))
+#                 print cache._records.Record
 
 #------------------------------------------------------------------------
 #
 # DESCRIBE NEW
 #
 #------------------------------------------------------------------------
-def do_describe_new():
+def do_describe_new(install=False):
     """Report on packages that are newly available.
 
     Arguments:
@@ -251,6 +333,9 @@ def do_describe_new():
         print "No new packages"
     else:
         do_describe(new_pkgs);
+        if install:
+            print "="*76
+            do_install(new_pkgs)
 
 #------------------------------------------------------------------------
 #
@@ -314,11 +399,11 @@ def do_force(packages):
             # the download archive, if there is any there.
             #
             lscmd  = "/bin/ls " + archives
-            lscmd += " | egrep '^" + pkg + "_' | sort | tail -1"
+            lscmd += " | egrep '^" + pkg + "_' | sort | tail -n -1"
             matches = perform.execute(lscmd, pipe=True)
             debpkg = matches.readline().strip()
             #
-            # If the package was not found then download it before
+            # If the package was not perfound then download it before
             # it is force installed.
             #
             if not debpkg:
@@ -364,11 +449,12 @@ def do_hold(packages):
 # INSTALL
 #
 #------------------------------------------------------------------------
-def do_install(packages):
+def do_install(packages, noauth=""):
     """Install packages.
 
     Arguments:
     packages    List of packages to be installed or path to deb file
+    noauth      A string which might be --allow-unathenticated
 
     Returns:"""
 
@@ -453,7 +539,7 @@ def do_install(packages):
 #                   (release, perform.concat(packages))
 #       perform.execute(command, root=1)
     else:
-        command = "apt-get install " + perform.concat(packages)
+        command = "apt-get %s install " % noauth + perform.concat(packages)
         perform.execute(command, root=1)
 
 #------------------------------------------------------------------------
@@ -461,11 +547,12 @@ def do_install(packages):
 # INSTALL SUGGEST
 #
 #------------------------------------------------------------------------
-def do_install_suggest(packages, type):
+def do_install_suggest(packages, type, noauth=""):
     """Install packages suggested by the listed packages.
 
     Arguments:
     packages    List of packages 
+    noauth      A string which might be --allow-unathenticated
 
     Returns:"""
 
@@ -480,16 +567,10 @@ def do_install_suggest(packages, type):
     # For each package obtain the list of suggested packages
     #
     suggest_list = ""
-    #
-    # As root obtain the details of each available package,
-    # using apt-cache (requires sudo). 23 Aug 2003.
-    #
-    # Remove root=1 from execute since apt-cache seems accessible to
-    # user.
-    #
-    command = "apt-cache dumpavail"
-    packages_pipe = perform.execute(command, noquiet=True, pipe=True)
-    avail = apt_pkg.ParseTagFile(packages_pipe)
+    
+    ## Obtain the details of each available package.
+
+    avail = get_available()
     #
     # Check for information in the Available list
     #
@@ -533,7 +614,7 @@ def do_install_suggest(packages, type):
     #else:
     #    cont = ""
     #if cont == "" or cont == "Y" or cont == "y" or cont == "yes":
-    command = "apt-get  --show-upgraded install " \
+    command = "apt-get %s --show-upgraded install " % noauth \
               + perform.concat(packages) + suggest_list
     perform.execute(command, root=1)
 
@@ -543,9 +624,7 @@ def do_install_suggest(packages, type):
 #
 #-----------------------------------------------------------------------
 def do_listsections():
-    command = "apt-cache dumpavail"
-    packages_pipe = perform.execute(command, noquiet=True, pipe=True)
-    avail = apt_pkg.ParseTagFile(packages_pipe)
+    avail = get_available()
     sections = []
     while avail.Step():
         s = avail.Section.get("Section")
@@ -554,9 +633,7 @@ def do_listsections():
     print "\n".join(sections)
 
 def do_listsection(pattern):
-    command = "apt-cache dumpavail"
-    packages_pipe = perform.execute(command, noquiet=True, pipe=True)
-    avail = apt_pkg.ParseTagFile(packages_pipe)
+    avail = get_available()
     while avail.Step():
         if (pattern == avail.Section.get("Section")):
             print avail.Section.get("Package")
@@ -599,6 +676,28 @@ def do_listnames(pattern, pipe=False):
     # End fix for Bug #292581
     return perform.execute(command, root=needsudo, pipe=pipe)
 
+def do_listscripts(pkg):
+    scripts = ["preinst", "postinst", "prerm", "postrm"]
+    if re.match(".*\.deb$", pkg):
+        command = "ar p " + pkg + " control.tar.gz | tar ztvf -"
+        pkgScripts = perform.execute(command, pipe=True).readlines()
+        for script in scripts:
+            if "./" + script in "".join(pkgScripts):
+                nlen = (72 - len(script))/2
+                print ">"*nlen, script, "<"*nlen
+                command = "ar p " + pkg + " control.tar.gz |" +\
+                          "tar zxvf - -O ./" + script +\
+                          " 2>/dev/null"
+                perform.execute(command)
+    else:
+        root = "/var/lib/dpkg/info/"
+        for script in scripts:
+            fname = root + pkg + "." + script
+            if os.path.exists(fname):
+                nlen = (72 - len(script))/2
+                print ">"*nlen, script, "<"*nlen
+                perform.execute("cat " + fname)
+
 #------------------------------------------------------------------------
 #
 # NEW
@@ -654,9 +753,14 @@ def do_news(packages):
 
     Returns: News items
     """
-    ping_host("packages.qa.debian.org")
+    orig = packages
+    ping_host("packages.debian.org")
     results = tempfile.mktemp()
     packages = map_sources(packages)
+    if not packages:
+        # list the original package name (maybe it is a source package)
+        # packages = map(lambda(x): [x,x], orig)
+        print "No package of that name found - perhaps it's a source package."
     for pk in packages:
         p = pk[0]
         #
@@ -664,23 +768,18 @@ def do_news(packages):
         #
         print "="*((70-len(p))/2) + " " + p + " " + "="*((70-len(p))/2)
         #
-        # Pipe through Lynx to map HTML entities like &quot;
-        # See comments in do_changelog for details about
-        # source packages with different versions.
         #
-        command  = "wget --timeout=60 --output-document=-" 
-        command += " http://packages.qa.debian.org/%s/" % (p[0])
-        command += p.split()[0]
-        command += "/news/1.html 2> /dev/null | lynx --dump --stdin > "
-        command += results
-        perform.execute(command)
-        command = "cat " + results + " | " +\
-                  "awk 'BEGIN{pr=0}/^Description:/{pr=1}/^Files:/{pr=0}" +\
-                  "pr==1{print;next}{next}' "
+        #
+        command  = "wget --timeout=60 --output-document=-"
+        command += " http://packages.debian.org/changelog:" + p.split()[0]
+        command += " 2> /dev/null "
+        #
+        # Strip out only the first entry in the changelog.
+        #
+        command += "| awk 'NR==1{print;next} /^[^ ]/{exit}{print;next}' "
         perform.execute(command)
         if os.path.exists(results): os.remove(results)
 
-
 #------------------------------------------------------------------------
 #
 # CHANGELOG
@@ -689,47 +788,50 @@ def do_news(packages):
 #
 #------------------------------------------------------------------------
 
-def pool_version(pkg, orig):
-    #
-    # Check first if package is available, if not then
-    # get version from original. Fixes Bug#295455
-    #
-    revert = False
-    changes.load_dictionaries()
-    available = changes.get_available_list()
-    if pkg not in available:
-        package = orig
-        revert = True
-    else:
-        package = pkg
-    #
-    # Get the version
-    #
-    command = "apt-cache show " + package
-    packages_pipe = perform.execute(command, noquiet=True, pipe=True)
-    Parse = apt_pkg.ParseTagFile(packages_pipe)
-    version = '0'
-    filename = ''
-    while Parse.Step():
-        v = Parse.Section.get("Version")
-        if perform.execute("dpkg --compare-versions %s gt %s" % (v, version))\
-               == 0:
-            version = v
-            filename = Parse.Section.get("Filename")
-            #
-            # If we reverted to the original package, be sure to get the
-            # changelog from the source package. Fixes Bug#295455
-            #
-            if revert:
-                filename = filename.replace(orig, pkg)
-            #
-            # Special case of local installed packages.
-            # Take a guess... Example is my own locally installed wajig.
-            # 
-            if filename[0:2] == "./":
-                filename = "pool/main/" + filename[2] +\
-                           filename.split("_")[0][1:] + "/" + filename[2:]
-    return (filename, version)
+# 2006-05-24 NO LONGER USED - COMMENT OUT FOR NOW THEN REMOVE ON NEXT
+# CLEANUP, JUST IN CASE...
+
+# def pool_version(pkg, orig):
+#     #
+#     # Check first if package is available, if not then
+#     # get version from original. Fixes Bug#295455
+#     #
+#     revert = False
+#     changes.load_dictionaries()
+#     available = changes.get_available_list()
+#     if pkg not in available:
+#         package = orig
+#         revert = True
+#     else:
+#         package = pkg
+#     #
+#     # Get the version
+#     #
+#     command = "apt-cache show " + package
+#     packages_pipe = perform.execute(command, noquiet=True, pipe=True)
+#     Parse = apt_pkg.ParseTagFile(packages_pipe)
+#     version = '0'
+#     filename = ''
+#     while Parse.Step():
+#         v = Parse.Section.get("Version")
+#         if perform.execute("dpkg --compare-versions %s gt %s" % (v, version))\
+#                == 0:
+#             version = v
+#             filename = Parse.Section.get("Filename")
+#             #
+#             # If we reverted to the original package, be sure to get the
+#             # changelog from the source package. Fixes Bug#295455
+#             #
+#             if revert:
+#                 filename = filename.replace(orig, pkg)
+#             #
+#             # Special case of local installed packages.
+#             # Take a guess... Example is my own locally installed wajig.
+#             # 
+#             if filename[0:2] == "./":
+#                 filename = "pool/main/" + filename[2] +\
+#                            filename.split("_")[0][1:] + "/" + filename[2:]
+#     return (filename, version)
 
 def do_changelog(packages):
     """List latest changelog for each package.
@@ -739,28 +841,36 @@ def do_changelog(packages):
 
     Returns: Changelogs
     """
-    # (gjw 30 Dec 2004) ping_host("changelogs.debian.net")
+    orig = packages
     ping_host("packages.debian.org")
     results = tempfile.mktemp()
     packages = map_sources(packages)
+    if not packages:
+        # list the original package name (maybe it is a source package)
+        # packages = map(lambda(x): [x,x], orig)
+        print "No package of that name found - perhaps it's a source package."
     for pk in packages:
-        #
-        # map_sources was changed 17 Feb 2005 to return pairs so that
-        # we can track the original package name in case the source
-        # package is not an installable package. Fixes Bug #295455.
-        #
+        
+        # map_sources was changed (gjw 17 Feb 2005) to return pairs
+        # (source package name, original package name) so that we can
+        # track the original package name in case the source package
+        # is not an installable package. Fixes Bug #295455.  (gjw 23
+        # Apr 2006) With the change to using the changelog:pkg syntax,
+        # we don't any longer really need this.
+
         p = pk[0]
         o = pk[1]
-        #
+
         # Print a header.
-        #
+
         print "="*((70-len(p))/2) + " " + p + " " + "="*((70-len(p))/2)
-        #
-        # Some source pakages have a different version which will
-        # be included in the string: e.g. "bash (2.05b-2-15)".
-        # We handle this case by using this version number rather than
-        # the packages installed version number.
-        #
+        sys.stdout.flush()
+
+        # Some source pakages have a different version which will be
+        # included in the actual packaname string: e.g. "bash
+        # (2.05b-2-15)".  We handle this case by using this version
+        # number rather than the package's installed version number.
+
         pkg = p.split()
         if len(pkg) > 1:
             ver = pkg[1][1:-1] # Remove the round brackets
@@ -768,30 +878,36 @@ def do_changelog(packages):
         else:
             pkg = pkg[0]
             ver = "`egrep '^%s ' %s | cut -d' ' -f2`" % (pkg, available_file)
-        #
+
         # Find pool file name and get latest version number.
-        # Previously I relied on changelogs.debian.net, but it seems that
-        # the symlink from, e.g., http://changelogs.debian.net/metacity
-        # to the actual changelog file does not get updated very quickly.
-        # So move to pointing the to latest version's actual changelog
-        # on packages.debian.org. (gjw 30 Dec 2004).
-        #
+        # Previously I relied on changelogs.debian.net, but it seems
+        # that the symlink from, e.g.,
+        # http://changelogs.debian.net/metacity to the actual
+        # changelog file does not get updated very quickly.  So move
+        # to pointing the to latest version's actual changelog on
+        # packages.debian.org. (gjw 30 Dec 2004).
+
         # Extra argument added 17 Feb for fix for Bug#295455. The orig
         # package name is used to get version if source pkg is not an
         # installable package.
+
+        # (gjw 23 Apr 2006) Move to using changelog:pkg
+        # syntax. Hopefully this takes care of the complications.
         #
-        (pool, version) = pool_version(pkg, o)
-        changelog_path = re.sub(r'_[^_]*\.deb$', '/changelog', pool)
-        #
+        # (pool, version) = pool_version(pkg, o)
+        # changelog_path = re.sub(r'_[^_]*\.deb$', '/changelog', pool)
+
         # Generate the command to retireve the changelog.
-        #
+
         command  = "wget --timeout=60 --output-document=-"
         # (gjw 30 Dec 2004) command += " http://changelogs.debian.net/"
-        command += " http://packages.debian.org/changelogs/"
-        #command += p[0] + "/"
-        #command += "%s/%s_%s/changelog 2> /dev/null" % (pkg, pkg, ver)
-        # (gjw 30 Dec 2004) command += "%s 2> /dev/null" % (pkg)
-        command += "%s 2> /dev/null" % (changelog_path)
+        command += " http://packages.debian.org/"
+        #command += "changelogs/" + p[0] + "/"
+        #command += "%s/%s_%s/changelog" % (pkg, pkg, ver)
+        # (gjw 30 Dec 2004) command += "%s" % (pkg)
+        # (gjw 23 Apr 2006) Try changelog: command += "changelogs/%s" % (changelog_path)
+        command += "changelog:" + p
+        command += " 2> /dev/null"
         perform.execute(command)
 
 #------------------------------------------------------------------------
@@ -799,7 +915,7 @@ def do_changelog(packages):
 # NEWUPGRADES
 #
 #------------------------------------------------------------------------
-def do_newupgrades():
+def do_newupgrades(install=False):
     """Report on packages that are newly upgraded.
 
     Arguments:
@@ -820,6 +936,9 @@ def do_newupgrades():
             print "%-24s %-24s %-24s" % (new_upgrades[i], \
                             changes.get_available_version(new_upgrades[i]), \
                             changes.get_installed_version(new_upgrades[i]))
+        if install:
+            print "="*74
+            do_install(new_upgrades)
 
 def do_removedepend(pkg, purge=False):
     print "Checking dependencies. Please wait..."
@@ -966,7 +1085,7 @@ def do_status(packages, snapshot=False):
         for l in fobj:
             print "=".join(l.split()[0:2])
     else:
-        perform.execute(command)
+        perform.execute(command, langC=True)
     #
     # Check whether the package is not in the installed list, and if not
     # list its status appropriately.
@@ -983,7 +1102,7 @@ def do_status(packages, snapshot=False):
             command = command +\
               " | awk '{printf(\"%-20s\\t%-15s\\t%-15s\\t%-15s\\n\", " +\
               "$1, \"N/A\", $2, $3)}'"
-            perform.execute(command)
+            perform.execute(command, langC=True)
     #
     # Tidy up - remove the "installed file"
     #
@@ -1199,6 +1318,8 @@ def do_whichpkg(filename):
     # Now obtain the information from the Debian server
     #
     results = tempfile.mktemp()
+    filename = filename.replace('+', '%2B')
+
     command = "wget --timeout=60 --output-document=" + results +\
               " http://" + packages_host + "/cgi-bin" +\
               "/search_contents.pl\?word=" + filename +\
@@ -1363,6 +1484,17 @@ def do_recdownload(packages):
     command = "apt-get --download-only --reinstall -u install " + perform.concat(packageNames)
     perform.execute(command, root=1)
 
+
+def versions(packages):
+    if len(packages) == 0:
+        command = "apt-show-versions"
+    else:
+        command = ""
+    for p in packages:
+        command += "apt-show-versions " + p + "; "
+    perform.execute(command)
+    
+
 def main():
     do_size([], 100)
 
