@@ -62,9 +62,11 @@ def usage():
         Options include:
         -d|--debug      Catch exceptions thrown by the program.
         -h|--help       Print this documenation and exit.
+        -n|--noauth     Allow packages from unathenticated archives.
         -q|--quiet	Do not show progress of commands.
         -s|--simulate   Show commands to be executed but don't execute.
         -t|--teaching   Show commands to be executed and then execute them.
+        -y|--yes        Assume yes for any questions asked.
 """
 
 #
@@ -86,6 +88,7 @@ def help(verbose):
  install        Install (or upgrade) one or more packages or .deb files
  remove         Remove one or more packages (see also purge)
 
+ toupgrade      List packages with newer versions available for upgrading
  upgrade        Upgrade all of the installed packages or just those listed
 
  listfiles      List the files that are supplied by the named package
@@ -153,15 +156,17 @@ Run LIST-COMMANDS for a complete list of commands.
  list-files     List the files that are supplied by the named package
  list-hold      List those packages on hold
  list-installed List packages (with optional argument substring) installed
- list-log       List the contents of the install/remove log file
+ list-log       List the contents of the install/remove log file (filtered)
  list-names     List all known packages or those containing supplied string
  list-orphans   List libraries not required by any installed package
+ list-scripts   List the control scripts of the package of deb file
  list-section   List packages that belong to a specific section
  list-section   List the sections that are available
  list-status    Same as list but only prints first two columns, not truncated
  list-wide      Same as list but avoids truncating package names
  local-dist-upgrade Dist-upgrade using packages already downloaded
  local-upgrade  Upgrade using packages already downloaded, but not any others
+ madison        Runs the madison command of apt-cache.
  move           Move packages in the download cache to a local Debian mirror
  new            List packages that became available since last update
  news           Obtain the latest news about the package
@@ -212,7 +217,10 @@ Run LIST-COMMANDS for a complete list of commands.
  unofficial     Search for an unofficial Debian package at apt-get.org
  update         Update the list of down-loadable packages
  update-alts    Update default alternative for things like x-window-manager
+ update-pci-ids Updates the local list of PCI ids from the internet master list
+ update-usb-ids Updates the local list of USB ids from the internet master list
  upgrade        Upgrade all of the installed packages or just those listed
+ versions       List version and distribution of (all) packages.
  whatis         A synonym for describe
  whichpkg       Find the package that supplies the given command or file 
 
@@ -220,9 +228,11 @@ Command line options:
 
  -h|--help      Print usage message.
  -q|--quiet     Do system commands everything quietly.
+ -n|--noauth    Allow packages from unathenticated archives.
  -s|--simulate  Trace but don't execute the sequence of underlying commands.
  -t|--teaching  Trace the sequence of commands performed.
  -v|--verbose=n Increase (or set) the level of verbosity (to n).
+ -y|--yes       Assume yes for any questions asked.
 
 Fuller documentation can be found at http://www.togaware.com/wajig.
 """
@@ -433,6 +443,7 @@ All wajig commands:
  list-files     List the files that are supplied by the named package
  list-hold      List those packages on hold
  list-installed List packages (with optional argument substring) installed
+ list-log       List the contents of the install/remove log file (filtered)
  list-names     List all known packages or those containing supplied string
  list-orphans   List libraries not required by any installed package
  list-section   List packages that belong to a specific section
@@ -497,10 +508,12 @@ All wajig commands:
 Command line options:
 
  -h|--help      Print usage message. 
+ -n|--noauth    Allow packages from unathenticated archives.
  -q|--quiet     Do system commands everything quietly. 
  -s|--simulate  Trace but don't execute the sequence of underlying commands. 
  -t|--teaching  Trace the sequence of commands performed. 
  -v|--verbose=n Increase (or set) the level of verbosity (to n). 
+ -y|--yes       Assume yes for any questions asked.
 
 Wajig expects a command and will call upon other Debian tools to
 perform the command.  Commands can be in mixed case and with hyphens
@@ -572,10 +585,23 @@ available on it using:
 
   $ wajig add-cd-rom  
 
+
+
 If you want to check when you last did an update then:
 
   $ wajig last-update
 
+There are quite a few archives available and you can test for a good
+connection to one with:
+
+  $ wajig netselect-apt
+
+This will write a candidate sources.list in the current directory,
+which you can then review and add to the system sources.list, if you
+wish, with
+
+  $ wajig edit-sources
+  
 
 FINDING PACKAGES
 
@@ -746,6 +772,13 @@ If this is an issue for you then you should use the upgrade command
 rather than dist-upgrade. This command will never remove or downgrade
 a package.
 
+To upgrade to a specific distribution (e.g., experimental) you can use:
+
+  # wajig dist-upgrade experimental
+
+Note that the mentioned distribution must also be mentioned in your
+/etc/apt/sources.list file.
+
 A neat trick with wajig is the ability to upgrade a collection of
 packages all with the same version number to another common version
 number:
@@ -769,6 +802,18 @@ To get rid of the configuation files as well use:
 
   $ wajig purge <package name>          (apt-get --purge remove)
 
+LOGGING THE INSTALLATION
+
+Whenever a package is installed, upgraded, or removed, a log is
+kept. Yo list the whole log:
+
+  $ wajig list-log
+
+or to list just those entries with a specifi string in their name:
+
+  $ wajig list-log biff
+
+
 CHECKING WHAT'S CHANGED BEFORE INSTALLING
 
 When you install an updated package it is sometimes useful to know
