@@ -48,28 +48,29 @@ match_commands = [] # For interactive command line completion
 def requires_no_args(command, args):
     if len(args) > 1:
         message = "no further arguments"
-        print progname + ": Error: " + command + " requires " + message
+        print progname + ": Error: " + command.upper() + " requires " + message
         finishup(1)
         return False
     return True
 
 def requires_one_arg(command, args, message):
     if len(args) != 2:
-        print progname + ": Error: " + command + " requires " + message
+        print progname + ": Error: " + command.upper() + " requires " + message
         finishup(1)
         return False
     return True
 
 def requires_opt_arg(command, args, message):
     if len(args) > 2:
-        print progname + ": Error: " + command + " requires " + message
+        print progname + ": Error: " + command.upper() +\
+              " has one optional arg: " + message
         finishup(1)
         return False
     return True
 
 def requires_args(command, args, type):
     if len(args) == 1: 
-        print progname + ": Error: " + command + " requires " + type
+        print progname + ": Error: " + command.upper() + " requires " + type
         finishup(1)
         return False
     return True
@@ -188,6 +189,8 @@ def interactive_shell():
 
 def main():
     global pause
+    global yes
+    global noauth
     #
     # Remove commas and insert the arguments appropriately.
     #
@@ -204,9 +207,9 @@ def main():
         #
         # Parse the command line arguments
         #
-        sopts = "dhpqstv"
+        sopts = "dhnpqstvy"
         lopts = ["debug", "help", "pause", "quiet","simulate","teaching",
-                 "verbose=", "version"]
+                 "verbose=", "version", "yes", "noauth"]
         opts, args = getopt.getopt(sys.argv[1:], sopts, lopts)
     except getopt.error, e:
         #
@@ -222,6 +225,8 @@ def main():
     teaching = 0
     verbose  = 0
     debug = False
+    yes = "" 
+    noauth = "" 
     #
     # Action the command line options
     #
@@ -242,6 +247,10 @@ def main():
         elif o in ("-t", "--teaching"):
             teaching = teaching + 1
             perform.set_teaching_level(teaching)
+        elif o in ("-y", "--yes"):
+            yes = " --yes "
+        elif o in ("-n", "--noauth"):
+            noauth = " --allow-unauthenticated "
         elif o == "-v":
             verbose = verbose + 1 
             commands.set_verbosity_level(verbose)
@@ -352,8 +361,12 @@ def select_command(command, args, verbose, teaching):
     elif command == "build":
         if requires_args(command, args, "a list of package names"):
             if requires_package("fakeroot", "/usr/bin/fakeroot"):
-                perform.execute("fakeroot " + "apt-get source -b " +
-                                perform.concat(args[1:]))
+                # First make sure dependencies are met
+                result = perform.execute("apt-get build-dep " +
+                                         perform.concat(args[1:]), root=1)
+                if not result:
+                    perform.execute("fakeroot " + "apt-get source -b " +
+                                    perform.concat(args[1:]))
 
     elif command == "builddepend":
         if requires_args(command, args, "a list of package names"):
@@ -375,11 +388,7 @@ def select_command(command, args, verbose, teaching):
 
     elif command == "dependents":
         if requires_one_arg(command, args, "package name"):
-            command = "apt-cache showpkg " + args[1] +\
-                      " | awk \'BEGIN{found=0}/^Reverse Depends:/{found=1;next}/^[^ ]/{found=0}found==1{print}{next}\' " +\
-                      " | sed \'s|Reverse Depends:||\' | tr \",\" \" \" " +\
-                      " | awk \'{print $1}\' | sort -u | grep -v \'^$\'"
-            perform.execute(command)
+            commands.do_dependents(args[1])
 
     elif command == "describe":
         if requires_args(command, args, "a list of packages"):
@@ -406,8 +415,13 @@ def select_command(command, args, verbose, teaching):
             commands.do_describe_new()
 
     elif command == "distupgrade":
-        if requires_no_args(command, args):
-            perform.execute("apt-get -u dist-upgrade", root=1)
+        if requires_opt_arg(command, args,
+                            "the distribution to upgrade to"):
+            cmd = "apt-get -u %s %s " % (yes, noauth)
+            if len(args) == 2:
+                cmd += "-t " + args[1] + " "
+            cmd += "dist-upgrade"
+            perform.execute(cmd, root=1)
 
     elif command == "download":
         if requires_args(command, args, "a list of packages"):
@@ -433,8 +447,9 @@ def select_command(command, args, verbose, teaching):
     elif command == "editsources" or \
          command == "setup":
         if requires_no_args(command, args):
-            if requires_package("base-config", "/usr/sbin/apt-setup"):
-                perform.execute("apt-setup", root=1)
+            # if requires_package("base-config", "/usr/sbin/apt-setup"):
+            #    perform.execute("apt-setup", root=1)
+            perform.execute("editor /etc/apt/sources.list", root=1)
 
     elif command == "filedownload" or \
          command == "downloadfile":
@@ -497,8 +512,9 @@ def select_command(command, args, verbose, teaching):
         #
         # Okay, so I'm sometimes dyslexic :-)
         #
-        if requires_args(command, args, "a list of packages, .deb files, or url"):
-            commands.do_install(args[1:])
+        if requires_args(command, args,
+                         "a list of packages, .deb files, or url"):
+            commands.do_install(args[1:], noauth)
 
     elif command == "installr" or \
          command == "recommended":
@@ -537,7 +553,7 @@ def select_command(command, args, verbose, teaching):
                             ", $6, $7, $8}' | sed 's|\.000000000||'")
 
     elif command == "list" or command == "listwide":
-        if requires_opt_arg(command, args, "at most one argument"):
+        if requires_opt_arg(command, args, "string to filter on"):
             cmd = ""
             if command == "listwide": cmd += "COLUMNS=200 " 
             cmd += "dpkg --list '*' | grep -v 'no description avail'"
@@ -547,7 +563,7 @@ def select_command(command, args, verbose, teaching):
             perform.execute(cmd)
 
     elif command == "listall":
-        if requires_opt_arg(command, args, "optional string to filter on"):
+        if requires_opt_arg(command, args, "string to filter on"):
             cmd = "apt-cache dumpavail |" +\
                             "egrep \"^(Package|Description): \" |" +\
                             "awk '/^Package: /{pkg=$2} /^Description: /" +\
@@ -564,7 +580,7 @@ def select_command(command, args, verbose, teaching):
                             "egrep -v '(\.1|\.1\.gz|\.8|\.8\.gz|README)$'")
 
     elif command == "listcache":
-        if requires_opt_arg(command, args, "optional string to filter on"):
+        if requires_opt_arg(command, args, "string to filter on"):
             cmd = "printf 'Found %d files %s in the cache.\n\n'\
             $(ls /var/cache/apt/archives/ | wc -l) \
             $(ls -sh /var/cache/apt/archives/ | head -1 | awk '{print $2}')"
@@ -608,20 +624,27 @@ def select_command(command, args, verbose, teaching):
             perform.execute("dpkg --get-selections | egrep 'hold$' | cut -f1")
 
     elif command == "listinstalled":
-        if requires_opt_arg(command, args, "at most one argument"):
+        if requires_opt_arg(command, args, "string to filter on"):
             commands.do_listinstalled(args[1:])
         
     elif command == "listlog":
-        if requires_no_args(command, args):
-            perform.execute("cat " + changes.log_file)
+        if requires_opt_arg(command, args, "string to filter on"):
+            cmd = "cat " + changes.log_file + "| sed 's|T| |'"
+            if len(args) == 2:
+                cmd = cmd + " | grep '" + args[1] + "'"
+            perform.execute(cmd)
 
     elif command == "listnames":
         # pdb.set_trace()
         if requires_opt_arg(command, args, "at most one argument"):
             result = commands.do_listnames(args[1:])
 
+    elif command == "listscripts":
+        if requires_one_arg(command, args, "a package name or deb file"):
+            result = commands.do_listscripts(args[1])
+            
     elif command == "liststatus":
-        if requires_opt_arg(command, args, "at most one argument"):
+        if requires_opt_arg(command, args, "package name"):
             cmd = "COLUMNS=400 " 
             cmd += "dpkg --list '*' | grep -v 'no description avail'"
             cmd += " | awk '{print $1,$2}'"
@@ -639,6 +662,9 @@ def select_command(command, args, verbose, teaching):
             perform.execute("apt-get --no-download --ignore-missing " \
                             + "--show-upgraded upgrade", root=1)
 
+    elif command == "madison":
+        perform.execute("apt-cache madison " + perform.concat(args[1:]))
+
     elif command == "move":
         if requires_no_args(command, args):
             perform.execute("apt-move update", root=1)
@@ -646,8 +672,16 @@ def select_command(command, args, verbose, teaching):
             perform.execute("apt-get clean",root=1)
 
     elif command == "new":
-        if requires_no_args(command, args):
-            commands.do_describe_new()
+        if requires_opt_arg(command, args, "whether to INSTALL the new pkgs"):
+            if len(args) == 1:
+                commands.do_describe_new()
+            elif args[1].lower() == "install":
+                commands.do_describe_new(install=True)
+            else:
+                print progname + ": Error: NEW only accepts optional " +\
+                      "argument INSTALL"
+                finishup(1)
+                return False
 
     elif command == "news":
         if requires_args(command, args, "a list of packages"):
@@ -655,8 +689,16 @@ def select_command(command, args, verbose, teaching):
                 commands.do_news(args[1:])
 
     elif command == "newupgrades" or command == "newupgrade":
-        if requires_no_args(command, args):
-            commands.do_newupgrades()
+        if requires_opt_arg(command, args, "whether to INSTALL upgraded pkgs"):
+            if len(args) == 1:
+                commands.do_newupgrades()
+            elif args[1].lower() == "install":
+                commands.do_newupgrades(install=True)
+            else:
+                print progname + ": Error: NEWUPGRADES only accepts " +\
+                      "optional argument INSTALL"
+                finishup(1)
+                return False
 
     elif command == "nonfree":
         if requires_no_args(command, args):
@@ -673,7 +715,7 @@ def select_command(command, args, verbose, teaching):
 
     elif command == "purge":
         if requires_args(command, args, "a list of packages"):
-            perform.execute("apt-get remove --purge " + perform.concat(args[1:]), root=1)
+            perform.execute("dpkg --purge " + perform.concat(args[1:]), root=1)
 
     elif command == "purgedepend":
         if requires_one_arg(command, args, "a single package"):
@@ -737,7 +779,8 @@ def select_command(command, args, verbose, teaching):
 
     elif command == "remove":
         if requires_args(command, args, "a list of packages"):
-            perform.execute("apt-get remove " + perform.concat(args[1:]), root=1)
+            perform.execute("apt-get %s remove %s" %
+                            (yes, perform.concat(args[1:])), root=1)
 
     elif command == "removedepend":
         if requires_one_arg(command, args, "a single package"):
@@ -807,6 +850,12 @@ def select_command(command, args, verbose, teaching):
 
     elif command == "source":
         if requires_args(command, args, "a list of package names"):
+            # First make sure dependencies are met
+            # John V. Belmonte 04 Nov 2005 requested this not be done
+            # Leave it to the user to do wajig builddepend
+            # It is not always necessary
+            #perform.execute("apt-get build-dep " + perform.concat(args[1:]),
+            #                root=1)
             perform.execute("apt-get source " + perform.concat(args[1:]))
 
     elif command == "start":
@@ -867,12 +916,22 @@ def select_command(command, args, verbose, teaching):
         if requires_one_arg(command, args, "name of alternative to update"):
             perform.execute("update-alternatives --config " + args[1], root=1)
 
+    elif command == "updatepciids":
+        if requires_package("pciutils", "/usr/bin/update-pciids"):
+            if requires_no_args(command, args):
+                perform.execute("update-pciids", root=1)
+        
+    elif command == "updateusbids":
+        if requires_package("usbutils", "/usr/sbin/update-usbids"):
+            if requires_no_args(command, args):
+                perform.execute("update-usbids", root=1)
+        
     elif command == "upgrade":
         if len(args) > 1:
             perform.execute("apt-get install " + perform.concat(args[1:]),
                                 root=1)
         else:
-            perform.execute("apt-get -u upgrade", root=1)
+            perform.execute("apt-get %s -u upgrade" % noauth, root=1)
 
     elif command == "upgradesecurity":
         sources_list = tempfile.mkstemp(".security", "wajig.", "/tmp")[1]
@@ -889,9 +948,13 @@ def select_command(command, args, verbose, teaching):
         perform.execute(command, root=1)
         if os.path.exists(sources_list): os.remove(sources_list)
 
-    elif command == "version":
-        documentation.version()
-
+    elif command == "version" or command == "versions":
+        if command == "version" and len(args) == 1:
+            documentation.version()
+        else:
+            if requires_package("apt-show-versions",
+                                "/usr/bin/apt-show-versions"):
+                commands.versions(args[1:])
     elif command == "whatis":
         if requires_args(command, args, "a list of package names"):
             commands.do_describe(args[1:])
